﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;
using System.Windows.Navigation;

namespace ActivitySample.Views
{
    public partial class HeavyAsyncWork : Page
    {

        private BackgroundWorker bw;

        public HeavyAsyncWork()
        {
            InitializeComponent();
            RealPI = Math.PI;
        }

        public long IterationCount
        {
            get { return (long)GetValue(IterationCountProperty); }
            set { SetValue(IterationCountProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IterationCount.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IterationCountProperty =
            DependencyProperty.Register("IterationCount", typeof(long), typeof(HeavyAsyncWork), new PropertyMetadata(5000000000L));




        public long RefreshInterval
        {
            get { return (long)GetValue(RefreshIntervalProperty); }
            set { SetValue(RefreshIntervalProperty, value); }
        }

        // Using a DependencyProperty as the backing store for RefreshInterval.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty RefreshIntervalProperty =
            DependencyProperty.Register("RefreshInterval", typeof(long), typeof(HeavyAsyncWork), new PropertyMetadata(1000000L));


        public double CalcPI
        {
            get { return (double)GetValue(CalcPIProperty); }
            set { SetValue(CalcPIProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CalcPI.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty CalcPIProperty =
            DependencyProperty.Register("CalcPI", typeof(double), typeof(HeavyAsyncWork), new PropertyMetadata(0d));



        public double RealPI
        {
            get { return (double)GetValue(RealPIProperty); }
            private set { SetValue(RealPIProperty, value); }
        }

        // Using a DependencyProperty as the backing store for RealPI.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty RealPIProperty =
            DependencyProperty.Register("RealPI", typeof(double), typeof(HeavyAsyncWork), new PropertyMetadata(0d));




        public bool IsWorking
        {
            get { return (bool)GetValue(IsWorkingProperty); }
            set { SetValue(IsWorkingProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsWorking.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsWorkingProperty =
            DependencyProperty.Register("IsWorking", typeof(bool), typeof(HeavyAsyncWork), new PropertyMetadata(false));



        // Executes when the user navigates to this page.
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            (Resources["LinkAnimation"] as Storyboard).Begin();
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            (Resources["LinkAnimation"] as Storyboard).Stop();
        }

        void bw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            IsWorking = false;
        }

        private void Calculate_Click(object sender, RoutedEventArgs e)
        {
            if (IsWorking)
                return;
            IsWorking = true;
            long iter = IterationCount;
            long refresh = RefreshInterval;
            bw = new BackgroundWorker();
            bw.WorkerSupportsCancellation = true;
            bw.DoWork += new DoWorkEventHandler((obj, args) =>
            {
                long inCount = 0;
                long total = 0;
                Random rand = new Random();
                for (int i = 0; i < iter && !((BackgroundWorker)obj).CancellationPending; i++)
                {
                    double x = rand.NextDouble();
                    double y = rand.NextDouble();
                    if (IsInQuarterCircle(x, y))
                        inCount++;
                    total++;
                    if (i % refresh == 0)
                    {
                        UpdateCalculatedPI(CalculatePI(inCount, total));
                    }
                }
                UpdateCalculatedPI(CalculatePI(inCount, total));
            });
            bw.RunWorkerCompleted += new RunWorkerCompletedEventHandler(bw_RunWorkerCompleted);
            bw.RunWorkerAsync();
        }

        private double CalculatePI(long inCount, long total)
        {
            return (4.0 * inCount / total);
        }

        private void UpdateCalculatedPI(double d)
        {
            Deployment.Current.Dispatcher.BeginInvoke(() =>
            {
                CalcPI = d;
            });
        }

        private static bool IsInQuarterCircle(double x, double y)
        {
            double result = Math.Sqrt(x * x + y * y);
            return result <= 1;
        }

        private void StopButton_Click(object sender, RoutedEventArgs e)
        {
            if (bw != null && bw.IsBusy)
                bw.CancelAsync();
        }
    }
}
